/*
***************************************************************************************
 AD9833 DDS Tone Burst Generator
 Version 2 - Add 1Hz resolution to both Burst and Continuous modes

 3/11/2022

 Copyright 2021-2022
 Jed Margolin

 Permission is granted to individuals to make copies for their personal use.
 Any commercial use requires my permission

 Uses gen9833 rev 3 board

 Uses MSP430G2402 or MSP430G2302

 Built with Code Composer Studio v10
 No crystal. Uses the factory DCO data
 Use with 0 - Register Optimization Only

 F/D Switch: Switch between Burst, Frequency, and Sweep Modes;
   Press and hold for about 2 seconds (then release) to change modes; Comes up in Burst Mode

 In Burst Mode:

  INC - Press and release to increase the time between bursts
      - Press and hold for about 2 seconds to keep increasing the time between bursts; release to stop

  DEC - Press and release to decrement the time Between bursts
      - Press and hold for about 2 seconds to keep decreasing the time between bursts; release to stop

  MODE - Press and release to toggle the time between bursts to either 1 ms or 10 ms

 In Frequency Mode:

  INC - Press and release to increase the frequency
      - Press and hold for about 2 seconds to keep increasing the frequency; release to stop

  DEC - Press and release to decrement the frequency
      - Press and hold for about 2 seconds to keep decreasing the frequency; release to stop

  MODE - Press and release to change frequency step: 1 Hz, 10 Hz, and 100 Hz

 In Sweep Mode: Sweeps 20 Hz to 10KHz until F/D is pressed

 Port 1:
   D0  input     SW1 (INC)
   D1  input     SW2 (DEC)
   D2  input     SW3 (MODE)
   D3  input     SW4 (F/D)
   D4  output    9833 /CS (FSYNC)
   D5  output    9833 SCLK
   D6  output    9833 SDIN
   D7  input     Slide Switch (not used)

 Port 2:
   D0  output    LCD D4
   D1  output    LCD D5
   D2  output    LCD D6
   D3  output    LCD D7
   D4  output    LCD /E (Enable)
   D5  output    LCD RS (Register Select)

   D6  output    Sync for oscilloscope          (XIN no crystal)
   D7  output    Flag for testing the software  (XOUT no crystal)
***************************************************************************************
*/

#include <msp430.h>
#include "main.h"

#define MSPTYPE  0     // 1 = MSP430G2553, 0 = MSP430G2402 or MSP430G2302
#define MAXTIME  500   // ms
//======================================================================
int main(void)
{
   WDTCTL = WDTPW + WDTHOLD;                 // Stop Watch Dog Timer

// No crystal. Use the factory default values
   DCOCTL = CALDCO_16MHZ;      // factory calibration values
   BCSCTL1 = CALBC1_16MHZ;

// Set up the Ports
// Port 1 - I/O
   P1SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
   P1SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 1 - inputs
   P1REN |= (BIT0|BIT1|BIT2|BIT3|BIT7);    // pullup/pulldown enabled
   P1OUT |= (BIT0|BIT1|BIT2|BIT3|BIT7);    //
   P1DIR &= ~(BIT0|BIT1|BIT2|BIT3|BIT7);   // In

// Port 1 - outputs
   P1DIR |= (BIT4|BIT5|BIT6);
   P1OUT &= ~(BIT4|BIT5|BIT6);

// Port 2 - I/O
   P2SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
   P2SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 2 outputs
   P2DIR |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);     //Out
   P2OUT &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 2 Input (not used here as an input)
//   P2REN |= BIT7;    // pullup/pulldown enabled
//   P2OUT |= BIT7;    //
//   P2DIR &= ~BIT7;   // In

// P1.4 is DDS-FSYNC
// P1.5 is DDS-SCLK    // Clocks on rising edge
// P1.6 is DDS-SDATA
// Initialize
   P1OUT |= ~BIT4;  //  FSYNC High
   P1OUT |= ~BIT5; //   SSCLK High

#if MSPTYPE
// The MSP430G2553 has 3 ports. The PDIP brings out only 2 of them
// Port 3 - inputs
   P3SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);  // In
   P3SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
   P3REN |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);    // pullup/pulldown enabled
   P3OUT |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);    //
   P3DIR &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);   // In
#endif

// Initialize the LCD
   lcd_reset();

   domsg(&msg1);        // "Jed's Tone Burst"
   domsg(&msg2);       //    "Generator"
   waitsec();
   clear_lcd(0x80);
   clear_lcd(0xC0);

// Start in Tone Burst Mode - Frequency
   mode = 0;         // tone burst mode
   burst_mode = 1;   // burst frequency
   state = 0;
   switch_time = 4;

   frequency = 1000;
   freq_step = 10;
   do_frequency(frequency);   // send it to the AD9833 and display it
   domsg(&msg9);   // "> "
   domsg(&msg6);   // " 10 Hz "

   time_step = 1;
   time_off = 50;
   lcd_wr_bcd(0xC8, time_off);  // display the time off period
   domsg(&msg3);        // " 1 ms"

   asm(" nop");    // for emulator

// Set up timer Interrupts - 1ms
   CCTL0 = CCIE;                      // CCR0 interrupt enabled
   CCR0 = 4000;                       // 1ms interrupts
   TACTL = TASSEL_2 | ID_2 | MC_1;    // SMCLK, contmode = count up to CCR0
   TACTL |= TACLR;          // clear timer Resets TAR, clock divider, count direction
   __bis_SR_register(GIE);            // Timer interrupts without LPM0
// ==============================================
   while (1)           // main loop
   {

// wait for timer_flag to go high, 1ms interrupt
      while(!timer_flag)  { }
      timer_flag = 0;

      if(mode == 0)   {burst_delay();}  // tone burst

// If not tone burst mode it is one of the other two modes

// 4 ms timing for switches
      if(--switch_time == 0)
      {
         switch_time = 4;                   // 4ms for debouncing switches
         do_switch(P1IN, BIT0, &switch1);   // PortIn, Bit mask, switch structure
         do_switch(P1IN, BIT1, &switch2);   // PortIn, Bit mask, switch structure
         do_switch(P1IN, BIT2, &switch3);   // PortIn, Bit mask, switch structure
         do_switch(P1IN, BIT3, &switch4);   // PortIn, Bit mask, switch structure  Delay/Frequency

// Switch 4 long press?
         if(switch4.countrun && (switch4.count <150) )
         {
            switch4.count++;
         }

         if(switch4.countrun && (switch4.count >=150) )  // we have a long press
         {
            mode = mode +1;    // next mode
            if(mode >2)  {mode = 0;}
            switch4.countrun = 0;     // turn it off

// we switched modes so refresh the display labels
            switch(mode)
            {
               case 0:    // tone burst
                 frequency = 1000;
                 do_frequency(frequency);   // send it to the AD9833 and display it
                 domsg(&msg9);   // "> "
                 domsg(&msg6);   // " 10 Hz "
                 lcd_wr_bcd(0xC8, time_off);  // display the time off period
                 domsg(&msg3);        // " 1 ms"
                 burst_mode = 1;   // burst frequency
                 freq_step = 10;
                 break;

               case 1:    // continuous frequency
                 do_frequency(frequency);   // send it to the AD9833 and display it
                 clear_lcd(0xC0);
                 break;

              case 2:    // sweep frequency
                  clear_lcd(0x80);
                  freq_step = 100;
                  domsg(&msg5);     // "Sweep "
                  break;

            }  // end of first switch(mode)

         }  // end of second switch(mode)

         if(switch4.release)   // stop and reset the counter
         {
            switch4.release = 0;
            switch4.count = 0;
            switch4.countrun = 0;
         }

// Do the modes: Tone Burst, Continuous, or Sweep

// tone burst
         if(mode == 0)
         {
           if(switch4.press)           // Toggle between burst delay and burst frequency
           {
              burst_mode ^= 0x01;
              switch4.press = 0;
              switch4.countrun = 1;

              if(burst_mode)
              {
                 domsg(&msg9);   // (x80, 2, "> ");
                 domsg(&msg10);   // (xC0, 2, "  ");
              }
              else
              {
                 domsg(&msg11);   // (xC0, 2, "> ");
                 domsg(&msg12);   // (x80, 2, "  ");
              }
           }  // end switch4 pressed

           switch(burst_mode)    // burst delay or burst frequency?
           {
              case 0:    // burst delay switches
                sw_burst_delay();
                break;

              case 1:    // burst frequency switches
                sw_burst_freq();
                break;
           }  // end of switch

         } // end of mode 0 (tone burst

// continuous tone
         if(mode == 1)
         {
           if(switch4.press)
           {
              clear_lcd(0xC0);
              switch4.press = 0;
              switch4.countrun = 1;
           }

           sw_burst_freq();   // just do frequency

         } // end of mode 1 (continuous tone)

// frequency sweep
         if(mode == 2)
         {
           if(switch4.press)
           {
              clear_lcd(0x80);
              switch4.press = 0;
              switch4.countrun = 1;
           }

           sweep_freq();

         } // end of mode 2 (frequency sweep)

      }  // end of 4ms timing

   }  // end while

}  // end main
//==========================================================
void burst_delay()
{
// 1 ms timing for tone bursts

   switch(state)
   {

      case 0:     // start the burst

        do_frequency_only(frequency);

        // length of the burst
        count = 11;      // 10ms

        if(frequency<1000)       // make a longer pulse below 1KHz
        {
           count = 21;
        }

        if(frequency<200)       // make a longer pulse below 200 Hz
        {
           count = 41;
        }

        if(frequency<50)       // make a longer pulse below 50 Hz
        {
           count = 121;
        }

        state=1;       // next mode

        break;


      case 1:

        if(--count == 0)       // end of burst, turn it off
        {
           do_dds(0x2100);     // control register - DAC disabled was 2180
           count = time_off;   // time off between bursts
           state = 2;           // next mode
           P2OUT |= BIT6;      // 'Scope trigger high
        }

        break;


      case 2:

        if(--count==0)        // end of off period between bursts
        {
           state = 0;         // next mode, start over
           P2OUT &= ~BIT6;   // 'Scope trigger low
        }

        break;
   }  // end of switch
}
//=========================================================
// Switches for burst delay
void sw_burst_delay()
{

// SW1
   if(switch1.press == 1)
   {
       time_off = time_off + time_step;
       if(time_off > MAXTIME)  {time_off = MAXTIME;}      // max off time

       switch1.press = 0;         // reset the flag
       switch1.count = 0;
       switch1.countrun = 1;      // the counter is now running

       lcd_wr_bcd(0xC9, time_off);  // display the new time off period
   }

   if(switch1.countrun && (switch1.count <200) )
   {
      switch1.count++;
   }

   if(switch1.countrun && (switch1.count >=200) )
   {
      switch1.count = 160;        // limit the count
      time_off = time_off + time_step;

      if(time_off > MAXTIME)  {time_off = MAXTIME;}
      lcd_wr_bcd(0xC9, time_off);   // display the new time off period
   }

   if(switch1.release)         // stop and reset the counter
   {
       switch1.release = 0;
       switch1.count = 0;
       switch1.countrun = 0;
   }

// SW2
   if(switch2.press)
   {
      time_off = time_off - time_step;

      if(time_off < 1)  {time_off = 1;}

      switch2.press = 0;         // reset the flag
      switch2.count = 0;
      switch2.countrun = 1;      // the counter is now running

      lcd_wr_bcd(0xC9, time_off);  // display the new time off period
    }

    if(switch2.countrun && (switch2.count <200) )
    {
       switch2.count++;
    }

    if(switch2.countrun && (switch2.count >=200) )
    {
       switch2.count = 160;        // limit the count

       time_off = time_off - time_step;

       if(time_off < 11)  {time_off = 11;}
       lcd_wr_bcd(0xC9, time_off);     // display the new time off period
     }

     if(switch2.release)         // stop and reset the counter
     {
        switch2.release = 0;
        switch2.count = 0;
        switch2.countrun = 0;
     }

// SW3
   if(switch3.press)
   {
      switch3.press = 0;

      // change the time step here
      if(time_step == 1)  {time_step = 10;}
      else
      {
         if(time_step == 10) {time_step = 1;}
      }

      if(time_step == 1)  {domsg(&msg3);}        // " 1 ms"
      if(time_step == 10)  {domsg(&msg4);}        // "10 ms"
   }

   if(switch3.release)   // stop and reset the counter
   {
      switch3.release = 0;
   }

}  // end of Burst Mode
//=========================
void burst_freq()
{
   do_frequency_only(frequency);
}
//==========================
// Switches for frequency
void sw_burst_freq()
{
// SW1
   if(switch1.press == 1)
   {
      frequency = frequency + freq_step;            // increment the frequency
      if(frequency >10000) {frequency = 10000;}
      do_frequency(frequency);

      switch1.press = 0;         // reset the flag
      switch1.count = 0;
      switch1.countrun = 1;      // the counter is running
    }

   if(switch1.countrun && (switch1.count <200) )
   {
      switch1.count++;
   }

   if(switch1.countrun && (switch1.count >=200) )
   {
      switch1.count = 160;        // limit the count

      frequency = frequency + freq_step;
      if(frequency >10000)
      {
         frequency = 10000;
      }

      do_frequency(frequency);  // and display it
   }

   if(switch1.release)         // stop and reset the counter
   {
      switch1.release = 0;
      switch1.count = 0;
      switch1.countrun = 0;
   }

// SW2
   if(switch2.press)
   {
      frequency = frequency - freq_step;        // decrement the frequency
      if(frequency < 20)  {frequency = 20;}
      do_frequency(frequency);  // and display it

      switch2.count = 0;
      switch2.press = 0;
      switch2.countrun = 1;
   }

   if( switch2.countrun && (switch2.count <200) )
   {
      switch2.count++;
   }

   if( switch2.countrun && (switch2.count >=200) )
   {
      switch2.count = 160;

      frequency = frequency - freq_step;
      if(frequency <20)
      {
         frequency = 20;
      }

      do_frequency(frequency);   // and display it
   }

   if(switch2.release)
   {
      switch2.release = 0;
      switch2.count = 0;
      switch2.countrun = 0;
   }

// SW3
   if(switch3.press)
   {
      switch3.count = 0;
      switch3.press = 0;
      switch3.countrun = 1;

// change the frequency step here
      switch(freq_step)    // burst delay or burst frequency?
      {
         case 100:
           freq_step = 10;
           domsg(&msg6);        // "10 Hz"
           break;

         case 10:
           freq_step = 1;
           domsg(&msg13);        // "1 Hz"
           break;

         case 1:
           freq_step = 100;
           domsg(&msg7);        // "100 Hz"
           break;
      }
   }  // end of sw3 press

}  // end of Frequency Mode
//==============================================
void sweep_freq(void)
{
   frequency = frequency + 10;
   if(frequency >10000)
   {
      frequency = 20;
      P2OUT &= (~BIT6);
   }

   if(frequency >1000)
   {
      P2OUT |= BIT6;
   }

   do_frequency(frequency);   // and display it

   asm(" nop");    // for emulator
}
//==============================================
// Timer A0 interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A (void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(TIMER0_A0_VECTOR))) Timer_A (void)
#else
#error Compiler not supported!
#endif
{
//   P2OUT ^= BIT6;      // 'Scope trigger
   timer_flag = 1;
}
//==========================================================
void wait(unsigned int time)
{
    volatile unsigned int i,j;

    for(i=time; i>0; i--)
    {
        for(j=382; j>0; j--);
        {
        }
    }

    return;
}
//==========================================================
void do_switch(unsigned char port, unsigned char bit, struct myswitch *sptr)    // PortIn, Bit, switch structure
{
//    unsigned char temp;

// Debounce and process the switch
// sptr points to the switch's data block
    sptr->debounce  = sptr->debounce <<1;    // shifts in a zero
    if(port & bit)        // a zero has already been shifted in
    {
       sptr->debounce++ ;      // so make LSB a one
    }

// Debounced state is either 0x00 or 0xFF
    if(sptr->debounce == 0x00)  {sptr->state = 0x00;}
    if(sptr->debounce == 0xFF)  {sptr->state = 0x01;}

// process the switch
    if(sptr->state != sptr->old)     // we have a change in switch state
    {
       if( (sptr->old == 0x01) && (sptr->state == 0x00) )  // we have a switch press
       {
          sptr->press = 0x01;
          sptr->count = 0;        // clear the switch counter
       }

       if( (sptr->old == 0x00) && (sptr->state== 0x01) )  // we have a switch release
       {
          sptr->release = 0x01;
          sptr->count = 0;        // clear the switch counter       }
       }

       sptr->old = sptr->state;     // the new state is now the old state
    }

// We now know if a switch has been pressed (and how long it has been pressed)
//    and if a switch has been released

    asm(" nop");    // for emulator
}
//===================================================
// wait 10 ms
void wait10ms(void)
{
   wait(41);
}
//==========================================================
void wait4(unsigned int time)        // 500 uS
{
    volatile unsigned int i,j;

    for(i=time; i>0; i--)
    {
        for(j=20; j>0; j--);
        {
        }
    }

    return;
}
//=================================================================
void do_dds(unsigned int data)
{
    unsigned int n;

    P1OUT &= (~BIT4);   // FSYNC low

    for(n=16; n>0; n--)
    {
       if(data & 0x8000)
           {P1OUT |= BIT6;}   // SDATA high
       else
           {P1OUT &= (~BIT6);}   // SDATA low
       data = data<<1;

       P1OUT &= (~BIT5);  // SCLK low
       P1OUT |= BIT5;  // SCLK high
    }

    P1OUT |= BIT4;   // FSYNC high

}
//====================================
void do_dds_data(unsigned long data)
{

   unsigned int data_int;

    data_int = (data & 0x3FFF | 0x4000);
    do_dds(data_int);   //  frequency register 0 LSB; db15 = 0; db14 = 1

    data_int = ( ((data>>14) & 0x3FFF))  | 0x4000;
    do_dds(data_int);        //  frequency register 0 MSB; db15 = 0; db14 = 1
}
//==========================================================
void lcd_init(void)
{
    lcd_reset();
}
//===================================
void lcd_reset(void)
{
/*
P2.0 - LCD D4
P2.1 - LCD D5
P2.2 - LCD D6
P2.3 - LCD D7

P2.4 - LCD E (Enable High, Write Data, Enable Low)
P2.5 - LCD RS (Register Select; RS low = Instruction, RS High = Data)
*/

// Wait at least 16 ms after power-up
    wait10ms();    // wait 10 ms
    wait10ms();    // wait 10 ms

    P2OUT &= (~BIT5);    // RS = 0

    lcd_wr4(0x03);      // write nybble and wait 200us
    wait10ms();

    lcd_wr4(0x03);      // write nybble and wait 200us
    lcd_wr4(0x03);      // write nybble and wait 200us
    lcd_wr4(0x02);      // write nybble and wait 200us
    lcd_wr_instr(0x28); //  2 lines, 5x7 dots
    lcd_wr_instr(0x08); //  display off
    lcd_wr_instr(0x01); //  display on

    wait10ms();

//    lcd_wr_instr(0x0F); //  display on, cursor on, blink on
    lcd_wr_instr(0x0c);  //  display on, cursor oFF, blink oFF

    lcd_wr_instr(0x06);  //  entry mode set: cursor increment, no shift
}
//===============================================================
// Do LCD Message
void domsg(const struct messages *sptr)
{
    unsigned char m,n;
    unsigned char data;

    m = sptr->pos;  // position
    lcd_wr_instr(m);

    m = sptr->numchars;  // numbers of chars

    for(n=0; n<m; n++)
    {
        data = sptr->msg[n];
        lcd_wr_data(data);
    }
}
//=================================================================
void lcd_wr4(unsigned int data)      // write nybble and wait 200us
{

//    P2.4 - LCD E (Enable High, Write Data, Enable Low)
//    P2.5 - LCD RS (Register Select; RS low = Instruction, RS High = Data)

    P2OUT |= BIT4;     // Enable High

    data &= 0x0F;
    P2OUT &= 0xF0;
    P2OUT |= data;     // write nybble

    P2OUT &= (~BIT4);  //  /E low

    wait4(20);         // 200 us
}
//======================
// LCD - Write 8-bit instruction
// P2.5 - LCD RS (Register Select; RS low = Instruction, RS High = Data)
void lcd_wr_instr(unsigned int data)
{
   unsigned int dsave;

   P2OUT &= (~BIT5);    // RS = 0

   dsave = data;
   dsave  = dsave >>4;       // high nybble
   lcd_wr4(dsave);  // write nybble and wait 200 us

   data &=0xF;      // low nybble4
   lcd_wr4(data);
}
//======================
// LCD - Write 8-bit data
// P2.5 - LCD RS (Register Select; RS low = Instruction, RS High = Data)
void lcd_wr_data(unsigned int data)
{
    unsigned int dsave;

    P2OUT |= BIT5;    // RS = 1

    dsave = data;
    dsave = dsave >>4;       // high nybble
    lcd_wr4(dsave);  // write nybble and wait 200 us

    data &=0xF;      // low nybble4
    lcd_wr4(data);
}
//==========================================================
void lcd_wr_bcd(unsigned int pos, unsigned int bin)
{

// First Convert Binary to ASCII
// Adapted from 8-bit microcontroller code using a DAA instruction

// unsigned int bin = 0x386; // 0x67 = 103d; 0x386 = 902d
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char char1, char2, char3;
   int i;
   unsigned char hcarry;

   for(i=0; i<12; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x800)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
//       bcd3 = bcd3 + 1;
//       hcarry = 1;
     }
//        else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd2 + 0x30;
   char2 = bcd1 + 0x30;
   char3 = bcd0 + 0x30;

// Display it
   if(char1 == '0')  { char1 = ' '; }    // remove leading '0'

   lcd_wr_instr(pos);
   lcd_wr_data(char1);
   lcd_wr_data(char2);
   lcd_wr_data(char3);
   lcd_wr_data(' ');
   lcd_wr_data('m');
   lcd_wr_data('s');

}
//====================================================================
void lcd_wr_bcd_long(unsigned int pos, unsigned long bin)
{
// First Convert Binary to ASCII
// Adapted from 8-bit microcontroller code with a DAA instruction

//bin = 12345678;    // test

unsigned char bcd0 = 0;
unsigned char bcd1 = 0;
unsigned char bcd2 = 0;
unsigned char bcd3 = 0;
unsigned char bcd4 = 0;
unsigned char bcd5 = 0;
unsigned char bcd6 = 0;
unsigned char bcd7 = 0;

//unsigned char char1, char2, char3, char4, char5, char6, char7, char8;
unsigned char char4, char5, char6, char7, char8;
int i;
unsigned char hcarry;

   for(i=0; i<32; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80000000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
   }
   else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
        else {hcarry = 0;}

//---------- BCD3
        bcd3 = bcd3<<1;
        bcd3 = bcd3 + hcarry;

       if(bcd3 >9)
       {
           bcd3 = bcd3 - 10;
           hcarry = 1;
       }
          else {hcarry = 0;}

//---------- BCD4
       bcd4 = bcd4<<1;
       bcd4 = bcd4 + hcarry;

       if(bcd4 >9)
       {
          bcd4 = bcd4 - 10;
          hcarry = 1;
       }
       else {hcarry = 0;}

//---------- BCD5
       bcd5 = bcd5<<1;
       bcd5 = bcd5 + hcarry;

       if(bcd5 >9)
       {
          bcd5 = bcd5 - 10;
          hcarry = 1;
       }
       else {hcarry = 0;}

//---------- BCD6
       bcd6 = bcd6<<1;
       bcd6 = bcd6 + hcarry;

       if(bcd6 >9)
       {
          bcd6 = bcd6 - 10;
          hcarry = 1;
       }
       else {hcarry = 0;}

//---------- BCD7
       bcd7 = bcd7<<1;
       bcd7 = bcd7 + hcarry;

       if(bcd7 >9)
       {
          bcd7 = bcd7 - 10;
          hcarry = 1;
        }
       else {hcarry = 0;}
   }

// Convert to ascii
//   char1 = bcd7 + 0x30;
//   char2 = bcd6 + 0x30;
//   char3 = bcd5 + 0x30;
   char4 = bcd4 + 0x30;
   char5 = bcd3 + 0x30;
   char6 = bcd2 + 0x30;
   char7 = bcd1 + 0x30;
   char8 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);

   if(char4 == '0')
   {
      char4 = ' ';

      if(char5 == '0')
      {
         char5 = ' ';

         if(char6 == '0')
         {
            char6 = ' ';
         }

      }

   }

   lcd_wr_data(char4);
   lcd_wr_data(char5);
   lcd_wr_data(char6);
   lcd_wr_data(char7);
   lcd_wr_data(char8);

   lcd_wr_data(' ');
   lcd_wr_data('H');
   lcd_wr_data('z');
}
//=================================================
void do_frequency(unsigned int freq)
{
    lfreq = freq * 351830;
    dds_data = lfreq>>15;       // divide by 32768
    do_dds(0x2100);             //  control register
    do_dds_data(dds_data);      //  frequency register 0 LSB; db15 = 0; db14 = 1;
    do_dds(0x2000);             //  exit reset

    lcd_wr_bcd_long(0x88, freq);
}
//=================================================
void do_frequency_only(unsigned int freq)
{
    lfreq = freq * 351830;
    dds_data = lfreq>>15;       // divide by 32768
    do_dds(0x2100);             //  control register
    do_dds_data(dds_data);      //  frequency register 0 LSB; db15 = 0; db14 = 1;
    do_dds(0x2000);             //  exit reset
}
//==================================================================
void clear_lcd(unsigned char pos)
{
    unsigned int n;

    lcd_wr_instr(pos);

    for(n=16; n>0; --n)
    { lcd_wr_data(' ');}

}
//===============================================
void waitsec()
{
   int n;

   for(n=200; n>0; n--)
   {wait10ms();}
}
//==============================================
